@@ -35,7 +35,7 @@ std::vector<int> cpp_are_possible_ancestors(Rcpp::IntegerVector t_inf,
                                             Rcpp::StringVector genotype,
                                             Rcpp::StringVector gen_tree,
                                             Rcpp::IntegerVector cluster,
-                                            size_t i) {
+                                            int delta, size_t i) {
   size_t n = cluster.size();
   std::vector<int> out;
   out.reserve(n);
@@ -59,7 +59,7 @@ std::vector<int> cpp_are_possible_ancestors(Rcpp::IntegerVector t_inf,
   if(gen_ref == "Not attributed"){
     for (size_t j = 0; j < n; j++) {
       j_clust = cluster[j]-1;
-      if (t_inf[j_clust] < ref_t_inf) { // offset
+      if (t_inf[j_clust] < ref_t_inf && t_inf[j_clust] > ref_t_inf - delta) { // offset
         out.push_back(j_clust+1);
       }
     }
@@ -68,7 +68,7 @@ std::vector<int> cpp_are_possible_ancestors(Rcpp::IntegerVector t_inf,
     // genotype as i and i's descendents can be a cluster
     for (size_t j = 0; j < n; j++) {
       j_clust = cluster[j]-1;
-      if (t_inf[j_clust] < ref_t_inf && 
+      if (t_inf[j_clust] < ref_t_inf && t_inf[j_clust] > ref_t_inf - delta && 
           (gen_tree[j_clust] == gen_ref 
              || gen_tree[j_clust] == "Not attributed")) { // offset
         out.push_back(j_clust+1);
@@ -119,12 +119,16 @@ Rcpp::List cpp_log_like_s(Rcpp::NumericVector population, Rcpp::NumericMatrix di
     for(k = 0; k<size_pop; k++){
       for(j = 0; j<size_pop; j++){
         if(j>k){
-          nb_move(k, j) = population_a[k]*pow(1+distance(j,k), b);
+          nb_move(k, j) = population_a[k]*pow(distance(j,k), b);
           nb_move(j, k) = nb_move(k,j) * population_a[j] / population_a[k];
           sum_pop[j] += nb_move(k, j);
           sum_pop[k] += nb_move(j, k);
         } else if(j == k){
-          nb_move(k, j) = population_a[k];
+          if(distance(j,k) == 0){
+            nb_move(k, j) = population_a[k];
+          } else{
+            nb_move(k, j) = population_a[k] * pow(distance(j,k), b);
+          }
           sum_pop[k] += nb_move(k, j);
         }
       }
@@ -350,7 +354,8 @@ Rcpp::IntegerVector cpp_find_local_cases(Rcpp::IntegerVector alpha,
 
 // [[Rcpp::interfaces(r, cpp)]]
 // [[Rcpp::export()]]
-Rcpp::List cpp_swap_cases(Rcpp::List param, Rcpp::IntegerVector cluster, int i) {
+Rcpp::List cpp_swap_cases(Rcpp::List param, Rcpp::IntegerVector cluster, 
+                          Rcpp::IntegerVector move_alpha, int i) {
   Rcpp::IntegerVector alpha_in = param["alpha"];
   Rcpp::IntegerVector t_inf_in = param["t_inf"];
   Rcpp::IntegerVector kappa_in = param["kappa"];
@@ -371,14 +376,8 @@ Rcpp::List cpp_swap_cases(Rcpp::List param, Rcpp::IntegerVector cluster, int i)
     return out;
   }
   
-  
-  // escape if ancestor of the case is imported, i.e. alpha[x-1] is NA
-  
   int x = alpha_in[i-1];
-  //if (alpha_in[x-1] == NA_INTEGER) {
-  // return out;
-  //}
-  
+
   
   // replace ancestries:
   // - descendents of 'i' become descendents of 'x'
@@ -386,9 +385,9 @@ Rcpp::List cpp_swap_cases(Rcpp::List param, Rcpp::IntegerVector cluster, int i)
   
   for (size_t j = 0; j < length_cluster; j++) {
     j_clust = cluster[j]-1;
-    if (alpha_in[j_clust] == i) {
+    if (alpha_in[j_clust] == i && move_alpha[j_clust] == TRUE) {
       alpha_out[j_clust] = x;
-    } else if (alpha_in[j_clust] == x) {
+    } else if (alpha_in[j_clust] == x && move_alpha[j_clust] == TRUE) {
       alpha_out[j_clust] = i;
     }
   }
